/* log.c,v 1.0 1993/10/21 19:11:52 NIST golmie */
/*
 *
 * $Log: log.c,v $
 * Revision 4.0  1996/08/23 20:02:27  tlin
 * bump to mickey mouse version
 *
 * Revision 3.0  1996/05/30 14:40:54  tlin
 * bump to 3.0; before Tk adition
 *
 * Revision 2.5  1996/05/30 10:04:29  tlin
 * use cisePort.h
 *
 * Revision 2.4  1996/05/29 16:20:20  tlin
 * dgb_write() redo
 *
 * Revision 2.3  1996/05/29 02:46:37  tlin
 * remove #include "simx.h"
 *
 * Revision 2.2  1996/05/29 01:24:54  tlin
 * remove inline version of log_param(c, p)
 *
 * Revision 2.1  1996/05/28 05:57:31  tlin
 * bump to v 2.1; before tcl/tk conversion
 *
 * Revision 0.1  1996/05/27 22:51:30  tlin
 * test
 *
 * Revision 0.0  96/04/26  15:34:25  goyal
 * Lab1
 * 
 * Revision 1.0  96/04/24  15:01:46  shivkuma
 * Lab 1
 * 
 * Revision 1.1  96/04/22  00:00:00  shivkuma
 * Version Before Putting in User-Friendly names
 * 
 * Revision 0.0  96/02/21  00:44:00  goyal
 * Initial Revision
 * 
 *
 *
 */

/* SHIV : cleaned up for CISE */

/*  'Cluster' to write happenings to a log file.
    Contains functions for two different types of log--debugging log
    parameter logs.  */


#define LOGSOURCE		/* So log.h knows this is the real thing */
#include "cisePort.h"
#include "sim.h"
#include "q.h"
#include "list.h"
#include "mempool.h"
#include "hash.h"
#include "pdu.h"
#include "event.h"
#include "log.h"




/**************** Debugging log functions */

static int dbg_level = 0;   /* Debugging level -- DBG_ERR or DBG_INFO */
int dbg_set_level(int level)
{
    int temp = dbg_level;
    dbg_level = level;
    return(temp);
}

/* Debugging log-- writes name of component along with message */
int dbg_write(int level, Component *c, char *format, ...)
{
    va_list args;
    va_start( args, format );
    
    if (!(dbg_level & level))
	return 0;
    /* First put the name into the string */
    if (c)	{
	fputs(c->co_name, stderr);
	putc(' ', stderr);
    }
    /* Add the time */
    fprintf(stderr, "%d ", ev_now());
    vfprintf( stderr, format, args );
    fputs("\n", stderr);
    fflush(stderr);

    va_end( args );
    return 0;
}



/************* Parameter logging */


static char *log_param_path = "";

void log_param_set_path(path)
     char *path;
{
  log_param_path = path;
}


/*
 * Changed to no longer create a seperate file for each parameter that is
 * logged.  When first called, it opens one file, and all of the logs
 * are written to that file.
 */

static FILE *log_fp = NULL;

static
log_param_open_file()
{
  char name[500];
  int accessible;

  /* Create file name by appending the PID to "sim_param."  */
  name[0] = '\0';
  /* Path first */
  if (strlen(log_param_path))  {
    strcpy(name, log_param_path);
    strcat(name, "/");
  }
  /* Put "sim_log." onto name */
  strcat(name, "sim_log.");

  /* PID */
  sprintf(name + strlen(name), "%d", getpid());

  /* Make sure that the file doesn't already exist */
  do {
    if ((accessible = _access(name, 00)) == 0)
      /* File already exists--add an 'a' to the file name */
      strcat(name, "a");
  } while (accessible == 0);

  /* Open it */
  if (!(log_fp = fopen(name, "w")))  {
      dbg_write(DBG_ERR, NULL,
		"log_param_create: Couldn't open file '%s' for writing", name);
      return(FALSE);
    }
  dbg_write(DBG_INFO, NULL, "log_param_create: opened log file '%s'", name);

  return(TRUE);
}

int log_param_create(c, p)
     Component *c;
     Param *p;
{
  static int num = 0;
   

  /* If the file isn't already open, open it. */
  if (!log_fp)
    if (!log_param_open_file())
      return(FALSE);

  /* Write a comment line to the file with the number associated with this
     parameter, the component name, and the parameter name. */
  if (!p->p_log)  {
    fprintf(log_fp, "# %d '%s' '%s'\n", ++num, c->co_name, p->p_name);
    p->p_log = num;
  }

  dbg_write(DBG_INFO, NULL, "log_param_create: started logging "
	    "parameter '%s' from component '%s', number %d",
	    p->p_name, c->co_name, num);

  return(TRUE);
}


void
log_param_close(p)
     Param *p;
{
}

/*Log the packet sequence number */
/* SHIV : CISE :void log_packet_sequence(c, p, pkt)
     Component *c;
     Param *p;
     Packet *pkt;
{
  if (p->p_flags & LogMask && pkt->pk_length)
    fprintf(log_fp, "%d %u %10d %10d %s\n",
            p->p_log, ev_now(), pkt->u.t.pk_seq,
            pkt->u.t.pk_seq + pkt->pk_length - 1,
            pkt->pk_source_socket.so_port->co_name);
}

*/


/* Log acks & windows. */
/* SHIV : CISE : void log_ack_sequence(c, p, una, window)
     Component *c;
     Param *p;
     unsigned una, window;
{
  if (p->p_flags & LogMask && p->p_log)
    fprintf(log_fp, "%d %u %u %u\n", p->p_log, ev_now(),
	    una, una + window);
}
*/

/* Set the change mask in the parameter, and also log it if necessary. */
void log_param(c, p)
     Component *c;
     Param *p; 
{
  

  p->p_flags |= ChangeMask;

  if (p->p_flags & LogMask && p->p_log)
    fprintf(log_fp, "%d %u %s\n", p->p_log, ev_now(),
	    (*p->p_make_short_text)(c, p));
}

/* Log packets to a file.  Generally useful only for looking for bugs in
   the simulator code--in any real simulation, the volume of data would
   be too big to be usable.
*/

static int packet_logging = FALSE;

void log_toggle_packet_logging()
{
  packet_logging = !packet_logging;
}


